/* DOFOR.prg v1.1 Mon Apr-24-1995  
   Purpose : create a DOS batch file to apply a command to a group of files
      with separate replaceable parameters for file name, extension, etc.

   Usage: DOFOR FileSpec "DOS Command" "Condition" > DOS.BAT
   
   Copyright (c) 1995 by the author:  
   Patrick O'Beirne B.Sc. M.A. MICS, Systems Consultant, Author, Lecturer
   Trainer and Developer in Clipper       TickIT trained ISO 9000 Auditor
   Systems Modelling Ltd,    Villa Alba,  Tara Hill,  Gorey,  Co. Wexford
   IRELAND. Tel +353-55-2294  Fax +353-55-22297   Compuserve: 100023,1304
   Email: pobeirne@iol.ie   WWWeb: http://www.iol.ie/~pobeirne/index.html

   This is placed in the public domain for free distribution.
   You may not distribute altered copies of this software.
   Please refer suggestions for changes to me at the above address.

   Code Checked with GROK (TM) JF Kaster
   Compile in Clipper /m /n /w /es1
   Link with the Clipper sample library for the functions
   LISTASARRAY, FILEPATH, FILEEXT  
   You could use FT_TEMPFIL() instead of TempFile() below if you prefer.
   
*/
/*
   This allows you to do something to all files with one extension
   and apply that to a different extension. Like in NDOS.
   Suppose that %f is the full pathname and %n is the name w/o ext...
   
   FOR %f in *.prg,*.ch DO IF EXIST %n.BAK REN %n.BAK %n.TMP
   FOR %f in *.mac DO KEYRPT %f > %n.RPT

   So we simply write a batch file that has an entry for each file:
   e.g. DO.BAT
   KEYRPT MYFILE.MAC > MYFILE.RPT
      

   Usage:
   DOFOR *.MAC "KEYRPT %F > %N.RPT" > rpt.bat
   Substitutable strings are shown below under USAGE.
   
   Version : DOFOR.prg Thu  04-20-1995  
   Added a third FOR parameter which applies a condition.
   Here, we delete all .TMP files more than 30 days old :
   
   DOFOR *.TMP "DEL %F" "%O > 30" > deltmp.bat
   
   There is a built-in function FileRead('Filename',offset,length)
   which allows us to check for file CONTENTS. That can be used to check
   headers to select certain file types where the extension is not
   definite enough. Here, we copy all .MOZ (Netscape cache files) which
   contain a GIF image to an image directory with the extension .GIF:
   
   DOFOR *.MOZ "COPY %F c:\image\%N.GIF" "fileread('%F',0,3)='GIF'" > DO.BAT
   
*/
#include "common.ch"
#include "fileio.ch"
#include "directry.ch"
#include "set.ch"

Procedure DOFOR( cWildSpec, cBatCmd, cCondition )
//local hBat, cBatName:="DO.BAT", nLen // now redirect to StdOut instead
local nFile, cUprCmd, cFilePath, cStuff, cUprCond, lwrite
local aFileSpec, cFileSpec, nDir, aDir, aDirAll:={}, aFile
local nPosBS, nPosDot
Set(_SET_DATEFORMAT,"yyyy-mm-dd")
#ifdef DEBUG
   Default cWildSpec to "*.*"
   //default cBatName to "DO"
   default cBatCmd to "Echo %f %n %e %p %s %d %t %u"
#else
   default cCondition to ""
   if pcount() < 2
      ? "Usage:"
      ? 'DOFOR Filespec "Command" "Condition"'
      ? "Where Filespec can be one or more wildcards separated by commas"
      ? " and Command can contain any of these parameters:"
      ? "   %F    File name  & .ext"
      ? "   %N    Name only"
      ? "   %E    .Ext only"
      ? "   %P    Path name e.g. DOS\"
      ? "   %S    Size in bytes without spaces"
      ? "   %D    File date   CCYYMMDD"
      ? "   %T    File time without colons  HHMMSS"
      ? "   %U    A unique file name from date/time"
      ? "Writes to the standard output substituting the parameters as shown."
      ? "Redirect to a file to save the results as a batch file."
      ? "Example:"
      ? [DOFOR *.MAC,*.KEY "KEYRPT %F > %N.RPT" > DO.BAT]
      wait

      ? "If a condition is specified, it is evaluated for each file "
      ? "and if TRUE, the command is written to the DO.BAT file."
      ? "The replaceable parameters (%F etc) are substituted literally."
      ? "The following extra parameters are available in the condition:"
      ? "   %O    Age of file in days"
      ? "   %A    Attributes of file (AHSRO)"
      ?
      ? "To test the contents of a file,"
      ? "you can use the function FileRead(filename,offset,length)"
      ? [DOFOR *.MOZ "COPY %F %N.GIF" "fileread('%F',0,3)='GIF'" > DO.BAT]
      ? "When testing strings, note that the condition, including any embedded"
      ? "text, is always forced to UPPERCASE before making the test."
      ?
      ? "Example to test for the age of a file:"
      ? [DOFOR *.TMP "DEL %F" "%O > 30" > deltmp.bat]
      Quit
   endif
#endif   
   aFileSpec := ListAsArray( cWildSpec )
   for nDir  := 1 to len( aFileSpec )
      cFileSpec := aFileSpec[nDir]
      cFilePath := FilePath( cFileSpec ) // Path w/o filename if any
      aDir := Directory( cFileSpec )
      if empty( aDir )
         ? 'No files matching "'+cFileSpec+'"'
      else
         for nFile := 1 to len( aDir )
            aadd( aDir[nFile], cFilePath )
            aadd( aDirAll, aDir[nFile] )
         next
      endif
   next
   if empty( aDirAll )
      ? 'No files matching "'+cWildSpec+'"'
      Quit
   endif
   /*
   cBatName := DfltExt( cBatName, ".BAT" )
   hBat := FCreate( cBatName, FC_NORMAL )
   if hBat == F_ERROR
      ? 'Cannot create Batch file "'+cBatName+'"'
      Quit
   endif
   */   
   for nFile := 1 to len( aDirAll )
      aFile := aDirAll[nFile]
      cUprCmd := upper( cBatCmd )
      cUprCond := upper( cCondition )  // Should it be uppercased?
      
      cUprCmd := StrTran( cUprCmd, "%F", aFile[F_NAME] )    // File Name+.ext
      cUprCond := StrTran( cUprCond, "%F", aFile[F_NAME] )    // File Name+.ext

       // next, extract name w/o extension
      cStuff := aFile[F_NAME]
      nPosBs := RAT("\", cStuff)
      nPosDot:= RAT(".", cStuff)
      if nPosDot > nposBs
         // full path name may contain dots as in "..\file.ext"
         cStuff := SUBSTR(cStuff, 1, nPosDot - 1)
      ENDIF
      cUprCmd := StrTran( cUprCmd, "%N", cStuff )           // File Name only
      cUprCond := StrTran( cUprCond, "%N", cStuff )           // File Name only

      cStuff := "."+FileExt(aFile[F_NAME])
      cUprCmd := StrTran( cUprCmd, "%E", cStuff )           // Extension
      cUprCond := StrTran( cUprCond, "%E", cStuff )           // Extension
      
      cStuff := if(empty(atail(aFile)), CurDir()+"\", atail(aFile) )
      // Path name added as last element of Directory array
      cUprCmd := StrTran( cUprCmd, "%P", cStuff )              // Path
      cUprCond := StrTran( cUprCond, "%P", cStuff )           // Path

      cStuff := ltrim(str(aFile[F_SIZE]))
      cUprCmd := StrTran( cUprCmd, "%S", cStuff )              // File Size
      cUprCond := StrTran( cUprCond, "%S", cStuff )           // File Size

      cStuff := dtos(aFile[F_DATE])
      cUprCmd := StrTran( cUprCmd, "%D", cStuff )              // File Date
      cUprCond := StrTran( cUprCond, "%D", cStuff )           // File Date

      cStuff := Substr(aFile[F_TIME],1,2) ;
               +Substr(aFile[F_TIME],4,2) ;
               +Substr(aFile[F_TIME],7,2)  
      cUprCmd := StrTran( cUprCmd, "%T", cStuff )              // File Time HHMMSS
      cUprCond := StrTran( cUprCond, "%T", cStuff )            // File Time HHMMSS

      if "%U" $ cUprCmd                                        // Unique File name wanted 
         cStuff := TempFile()                               // Create unique file
         cUprCmd := StrTran( cUprCmd, "%U", cStuff )           // Unique file name
      endif      

      // Specials for Condition
      cUprCond := StrTran( cUprCond, "%A", aFile[F_ATTR])
      cUprCond := StrTran( cUprCond, "%O",   ;
                  ltrim(str(date()-aFile[F_DATE])))           // File Age days
                  
      
      if empty( cUprCond )
         lwrite := .T.
      else
         cStuff := type( cUprCond ) 
         if cStuff == "L" .or. cStuff == "UI" // Allow UDF such as fileread()
            lWrite := &(cUprCond)
         else
            OutStd( "REM: Condition not logical: "+cUprCond+chr(13)+chr(10))
            lWrite := .F.
         endif
      endif
      
      if lWrite
         OutStd( cUprCmd+chr(13)+chr(10) )
         /*
         nLen := len( cUprCmd ) + 2 // for Fwrite test next
         if Fwrite( hBat, cUprCmd+chr(13)+chr(10), nLen ) < nLen
            ? "Error",Ferror(),"Writing",cBatName
            Quit
         endif
         */
      endif
      
   next
   //Fclose( hBat)
return
   

// Translation function for Nanfor lib
Function StoD( ccyymmdd ) // --> returns Clipper date from string
//return FT_STOD( ccyymmdd )
// This ASSUMES SET DATE FORMAT above YYYY-MM-DD
return ctod( substr(ccyymmdd,1,4)+"-"  ;
            +substr(ccyymmdd,5,2)+"-"  ;
            +substr(ccyymmdd,7,2))   
            
// FileRead function like memoread but with offset and length
Function FileRead( cFileName, nOffset, nLength ) // --> Contents, max 63K
Static csString := ""
Static csFileName := ""
Static nsFileHandle := -1
Local nBytes := 0
if csFileName == cFileName // same file asked for as last time
   // save time by not reopening
   
else
   if nsFileHandle > -1     // close prev handle if open
      fclose( nsFileHandle )
   endif
   if (nsFileHandle := fopen( cFileName, FO_READ )) == -1   // Error if -1
      OutStd( "REM: Error "+ltrim(str(Ferror()))+" attempting to open "+cFileName+chr(13)+chr(10))
      csFileName := ""
   else
      csFileName := cFileName
   endif
endif
if nsFileHandle > -1
   default nOffset to 0
   default nLength to fseek( nsFileHandle, 0, FS_END ) // Seek to EOF for filesize
   if nLength > 64000   // max Clipper string length is 65000, but be safe.
      nLength := 64000
   endif
   Fseek( nsFileHandle, nOffset, FS_SET )       // go to offset posn
   csString := space( nLength )
   nBytes := fread( nsFileHandle, @csString, nLength )
   if nBytes < nLength  // unexpected
      csString := substr( csString, 1, nBytes)
   endif
endif

return csString

Function TempFile( cPath, cExtn, lCreate )   // -> MDDHMMSS.tt
// returns a new file Name from current time + Extension passed
// in given directory. Ext is the fractions of a sec if not specified
local cFileName, nHandle, dDate, nTime
   if cPath == NIL;      cPath := "";   endif
   if lCreate == NIL;   lCreate := .F.;   endif
   Do While .T.      // keep trying until we get a unique file
      dDate := date()
      nTime := seconds()
      cFileName := cPath+chr(64+month(dDate))   ;           // Month Jan=A to Dec=L
                  +right(dtos(dDate),2)   ;           // Day 2 digits
                  +chr(65+int(nTime/3600))         ;  // Hour 00=A to 23=X
                  +padl(ltrim(str(int(nTime/60 % 60))),2,"0")  ;  // Mins 2 digits
                  +padl(ltrim(str(int(nTime    % 60))),2,"0")     // Secs 2 digits
      if cExtn == NIL
         cFileName += ltrim(str(nTime-int(nTime),5,3))   // ext is hundredths secs
      else
         cFileName += iif('.' $ cExtn, '', '.') + cExtn  // specified ext
      endif
      if (nHandle := Fopen( cFileName )) != -1
         Fclose(nHandle)
      else
         if lCreate  // create the file just to reserve the name (multiuser)
            nHandle := FCreate( cFileName )
            Fclose( nHandle )
         endif
         exit // this file name does not exist (yet) so return it
      endif
   Enddo
Return (cFileName)


